@@ -12,7 +12,7 @@ import itertools
 import json
 import junky
 from junky.dataset import CharDataset, DummyDataset, FrameDataset, \
-                          LenDataset, TokenDataset
+                          LabelDataset, LenDataset, TokenDataset
 from mordl import WordEmbeddings
 from morra.base_parser import BaseParser
 from mordl.defaults import BATCH_SIZE, CONFIG_ATTR, CONFIG_EXT, LOG_FILE, \
@@ -147,9 +147,13 @@ class BaseTagger(BaseParser):
                 ds.add('t_{}'.format(i), t, with_lens=False)
 
         if labels:
-            y = TokenDataset(labels, pad_token='<PAD>', transform=True,
-                             keep_empty=False)
-            ds.add('y', y, with_lens=False)
+            if isinstance(labels[0], tuple):
+                y = TokenDataset(labels, pad_token='<PAD>', transform=True,
+                                 keep_empty=False)
+                ds.add('y', y, with_lens=False)
+            else:
+                y = LabelDataset(labels, transform=True, keep_empty=False)
+                ds.add('y', y)
 
         if for_self:
             self._ds = ds
@@ -167,7 +171,8 @@ class BaseTagger(BaseParser):
                 if not WordEmbeddings.transform(
                     ds_, sentences, batch_size=batch_size,
                     loglevel=0 if log_file is None else
-                             1
+                             1 if log_file == sys.stdin else
+                             2
                 ):
                     ds_.transform(sentences)
             elif typ == 't':
@@ -419,7 +424,7 @@ class BaseTagger(BaseParser):
                     batch = junky.to_device(batch, device)
                     with torch.no_grad():
                         pred = self._model(*batch)
-                    _, pred_indices = pred.max(2)
+                    pred_indices = pred.argmax(-1)
                     preds.extend(pred_indices.cpu().numpy().tolist())
                 values = ds_y.reconstruct(preds)
                 if with_orig:
@@ -527,8 +532,9 @@ class BaseTagger(BaseParser):
             return n, c, nt, ct, ca, ce, cr
 
         n = c = nt = ct = ca = ce = cr = ntok = ctok = ntokt = ctokt = 0
-        i = -1
-        for i, sentences in enumerate(corpora):
+        nsent = csent = 0
+        for nsent, sentences in enumerate(corpora, start=1):
+            csent_ = 1
             for gold_token, test_token in zip(*sentences):
                 wform = gold_token['FORM']
                 if wform and '-' not in gold_token['ID']:
@@ -556,6 +562,7 @@ class BaseTagger(BaseParser):
                                             n, c, nt, ct, ca, ce, cr)
                                 if c_ == c:
                                     ctok_ = 0
+                                    csent_ = 0 
                                 else:
                                     c = c_
                             ntokt += 1
@@ -564,21 +571,26 @@ class BaseTagger(BaseParser):
                         else:
                             ctok += 1
                     elif not (isgold or istest):
-                        n, c, nt, ct, ca, ce, cr = \
+                        n, c_, nt, ct, ca, ce, cr = \
                             compare(gold_label, test_label,
                                     n, c, nt, ct, ca, ce, cr)
+                        if c_ == c:
+                            csent_ = 0 
+                        else:
+                            c = c_
                     else:
                         raise TypeError(
                             'Inconsistent field types in gold and test '
                             'corpora'
                         )
+            csent += csent_
         if test:
             try:
                 next(gold)
             except StopIteration:
                 pass
         if log_file:
-            if i < 0:
+            if nsent <= 0:
                 print('Nothing to do!', file=log_file)
             else:
                 sp = ' ' * (len(header) - 2)
@@ -613,6 +625,8 @@ class BaseTagger(BaseParser):
                                           if ntok else
                                       '',
                                       c / n if n > 0 else 1.), file=log_file)
+                print('[By sentence accuracy: {}]'
+                          .format('{}'.format(csent / nsent)), file=log_file)
         return ct / nt if nt > 0 else 1.
 
     def train(self, field, add_fields, model_class, tag_emb_names, save_as,
@@ -867,7 +881,8 @@ class BaseTagger(BaseParser):
             print('\nMODEL CREATION', file=log_file)
         ds_ = ds_train.get_dataset('y')
         model_args = [len(ds_.transform_dict)]
-        model_kwargs['labels_pad_idx'] = ds_.pad
+        if hasattr(ds_, 'pad'):
+            model_kwargs['labels_pad_idx'] = ds_.pad
         if word_emb_type:
             ds_ = ds_train.get_dataset('x')
             model_kwargs['vec_emb_dim'] = ds_.vec_size
@@ -951,43 +966,45 @@ def conll18_ud_eval(gold_file, system_file, verbose=True, counts=False):
     [CoNLL18 UD Shared Task](https://universaldependencies.org/conll18/results.html)
     evaluation script.
 
-    positional arguments:<br/>
-      gold_file      Name of the CoNLL-U file with the gold data.<br/>
-      system_file    Name of the CoNLL-U file with the predicted.
+    Args:
 
-    optional arguments:<br/>
-      verbose        Print all metrics.<br/>
-      counts         Print raw counts of correct/gold/system/aligned words
-                     instead of prec/rec/F1 for all metrics.
+    **gold_file**: Name of the CoNLL-U file with the gold data.
 
-    If `verbose=False`, only the official CoNLL18 UD Shared Task
-    evaluation metrics are printed.
+    **system_file**: Name of the CoNLL-U file with the predicted.
+
+    **verbose** (`bool`): Print all metrics.
+
+    **counts** (`bool`): Print raw counts of correct/gold/system/aligned words
+    instead of prec/rec/F1 for all metrics.
+
+    If `verbose=False`, only the official CoNLL18 UD Shared Task evaluation
+    metrics are printed.
 
     If `verbose=True` (default), more metrics are printed (as precision,
     recall, F1 score, and in case the metric is computed on aligned words
-    also accuracy on these):<br/>
-      - Tokens: how well do the gold tokens match system tokens<br/>
-      - Sentences: how well do the gold sentences match system sentences<br/>
-      - Words: how well can the gold words be aligned to system words<br/>
-      - UPOS: using aligned words, how well does UPOS match<br/>
-      - XPOS: using aligned words, how well does XPOS match<br/>
-      - UFeats: using aligned words, how well does universal FEATS match<br/>
-      - AllTags: using aligned words, how well does UPOS+XPOS+FEATS match<br/>
-      - Lemmas: using aligned words, how well does LEMMA match<br/>
-      - UAS: using aligned words, how well does HEAD match<br/>
-      - LAS: using aligned words, how well does HEAD+DEPREL(ignoring subtypes)
-          match<br/>
-      - CLAS: using aligned words with content DEPREL, how well does
-          HEAD+DEPREL(ignoring subtypes) match<br/>
-      - MLAS: using aligned words with content DEPREL, how well does
-          HEAD+DEPREL(ignoring subtypes)+UPOS+UFEATS+FunctionalChildren(DEPREL+UPOS+UFEATS)
-          match<br/>
-      - BLEX: using aligned words with content DEPREL, how well does
-          HEAD+DEPREL(ignoring subtypes)+LEMMAS match
+    also accuracy on these):
+    - Tokens: how well do the gold tokens match system tokens.
+    - Sentences: how well do the gold sentences match system sentences.
+    - Words: how well can the gold words be aligned to system words.
+    - UPOS: using aligned words, how well does UPOS match.
+    - XPOS: using aligned words, how well does XPOS match.
+    - UFeats: using aligned words, how well does universal FEATS match.
+    - AllTags: using aligned words, how well does UPOS+XPOS+FEATS match.
+    - Lemmas: using aligned words, how well does LEMMA match.
+    - UAS: using aligned words, how well does HEAD match.
+    - LAS: using aligned words, how well does HEAD+DEPREL(ignoring subtypes)
+    match.
+    - CLAS: using aligned words with content DEPREL, how well does
+    HEAD+DEPREL(ignoring subtypes) match.
+    - MLAS: using aligned words with content DEPREL, how well does
+    HEAD+DEPREL(ignoring subtypes)+UPOS+UFEATS+FunctionalChildren(DEPREL+UPOS+UFEATS)
+    match.
+    - BLEX: using aligned words with content DEPREL, how well does
+    HEAD+DEPREL(ignoring subtypes)+LEMMAS match.
 
     If `count=True`, raw counts of correct/gold_total/system_total/aligned
-      words are printed instead of precision/recall/F1/AlignedAccuracy for all
-      metrics."""
+    words are printed instead of precision/recall/F1/AlignedAccuracy for all
+    metrics."""
     argv = sys.argv
     sys.argv[1:] = [gold_file, system_file]
     if verbose:
