@@ -22,8 +22,10 @@ class FeatTagger(BaseTagger):
 
     **feat** (`str`): the name of the feat which needs to be predicted by the
     tagger. May contains prefix, separated by a colon (`:`). In that case, the
-    prefix treat as a field name. Otherwise, we get `'FEATS'` as a field name.
-    Examples: `'Animacy'`; `'MISC:NE'`.
+    prefix treat as a field name. Also, if **feat** starts with a colon, we get
+    `'FEATS'` as a field name. Otherwise, if **feat** contain no colon, the
+    field name set to **feat** inself. Examples: `'MISC:NE'`; `':Animacy'`;
+    `'DEPREL'`.
 
     **feats_prune_coef** (`int`): feature prunning coefficient which allows to
     eliminate all features that have a low frequency. For each UPOS tag, we
@@ -38,13 +40,15 @@ class FeatTagger(BaseTagger):
     **NB**: the argument is relevant only if **feat** is not from FEATS field.
     """
     def __init__(self, feat, feats_prune_coef=6):
+        assert feat != 'UPOS', 'ERROR: for UPOS field use UposTagger class'
         super().__init__()
-        if feat.find(':') == -1:
-            feat = 'FEATS:' + feat
+        if feat[0] == ':':
+            feat = 'FEATS' + feat
         self._field = feat
         if feat.startswith('FEATS:'):
             feats_prune_coef = 0
         self._feats_prune_coef = feats_prune_coef
+        self._model_class = FeatTaggerModel
 
     def _transform_upos(self, corpus, key_vals=None):
         rel_feats = {}
@@ -163,9 +167,6 @@ class FeatTagger(BaseTagger):
 
         if self._feats_prune_coef != 0:
             kwargs['save_to'] = None
-
-            cdict = self._cdict
-
             key_vals = self._ds.get_dataset('t_0').transform_dict
             corpus = self._get_corpus(corpus, asis=True, log_file=log_file)
             corpus = self._transform_upos(corpus, key_vals=key_vals)
@@ -359,6 +360,8 @@ class FeatTagger(BaseTagger):
             self._cdict = CorpusDict(corpus=self._train_corpus,
                                      format='conllu_parsed', log_file=log_file)
             self._save_cdict(save_as + '.cdict.pickle')
+            if log_file:
+                print(file=log_file)
 
             list(self._transform_upos(self._train_corpus))
             key_vals = set(x['UPOS'] for x in self._train_corpus for x in x
@@ -366,5 +369,5 @@ class FeatTagger(BaseTagger):
                                             and '-' not in x['ID'])
             list(self._transform_upos(self._test_corpus, key_vals))
 
-        return super().train(self._field, 'UPOS', FeatTaggerModel, 'upos',
+        return super().train(self._field, 'UPOS', self._model_class, 'upos',
                              *args, **kwargs)
